// Generated by CoffeeScript 1.8.0
(function() {
  var CONN_CHECK, DefaultPort, EventEmitter, HandshakeTimeout, LRWebSocketConnection, LRWebSocketServer, PROTOCOL_7, Parser, Path, Url, fs, http, wsio, _connections, _nextConnectionId, _ref,
    __hasProp = {}.hasOwnProperty,
    __extends = function(child, parent) { for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; } function ctor() { this.constructor = child; } ctor.prototype = parent.prototype; child.prototype = new ctor(); child.__super__ = parent.prototype; return child; };

  wsio = require('websocket.io');

  http = require('http');

  Url = require('url');

  Path = require('path');

  fs = require('fs');

  EventEmitter = require('events').EventEmitter;

  _ref = require('./protocol'), Parser = _ref.Parser, PROTOCOL_7 = _ref.PROTOCOL_7, CONN_CHECK = _ref.CONN_CHECK;

  HandshakeTimeout = 1000;

  DefaultPort = 35729;

  _connections = {};

  _nextConnectionId = 1;

  LRWebSocketConnection = (function() {
    function LRWebSocketConnection(server, socket, id) {
      this.server = server;
      this.socket = socket;
      this.id = id;
      this.socket.on('message', this._ondata.bind(this));
      this.socket.on('close', this._onclose.bind(this));
      this.socket.on('error', this._onerror.bind(this));
      this.parser = new Parser;
      this.parser.on('connected', this._onHandshakeDone.bind(this));
      this.parser.on('oldproto', this._onoldproto.bind(this));
      this.parser.on('message', this._oncommand.bind(this));
      this.parser.on('error', this._onerror.bind(this));
      this.handshakeTimeout = setTimeout(this._onHandshakeTimeout.bind(this), HandshakeTimeout);
    }

    LRWebSocketConnection.prototype.send = function(command) {
      var payload;
      payload = JSON.stringify(command);
      LR.log.fyi("" + this.id + ": Sending message " + payload);
      return this.socket.send(payload);
    };

    LRWebSocketConnection.prototype.isMonitoring = function() {
      return !!this.protocols.monitoring;
    };

    LRWebSocketConnection.prototype._ondata = function(payload) {
      LR.log.fyi("" + this.id + ": Got message " + payload);
      return this.parser.process(payload);
    };

    LRWebSocketConnection.prototype._onclose = function(e) {
      this._cancelHandshakeTimeout();
      this.server._onwsdisconnected(this);
      return LR.log.fyi("" + this.id + ": Connection closed.");
    };

    LRWebSocketConnection.prototype._onerror = function(err) {
      LR.log.wtf("" + this.id + ": Web Socket communication error: " + err.message);
      return this.socket.close();
    };

    LRWebSocketConnection.prototype._onoldproto = function() {
      LR.log.wtf("" + this.id + ": Web Socket: old protocol v6 connection detected");
      this.socket.close();
      return this.server._onwsoldproto(this);
    };

    LRWebSocketConnection.prototype._onHandshakeTimeout = function() {
      this.handshakeTimeout = null;
      LR.log.wtf("" + this.id + ": Web Socket handshake timeout");
      return this.socket.close();
    };

    LRWebSocketConnection.prototype._cancelHandshakeTimeout = function() {
      if (this.handshakeTimeout) {
        clearTimeout(this.handshakeTimeout);
        return this.handshakeTimeout = null;
      }
    };

    LRWebSocketConnection.prototype._onHandshakeDone = function(protocols) {
      this.protocols = protocols;
      this._cancelHandshakeTimeout();
      this.send({
        command: "hello",
        protocols: [PROTOCOL_7, CONN_CHECK],
        serverName: "LiveReload 2"
      });
      this.server._onwsconnected(this);
      return LR.log.fyi("" + this.id + ": Web Socket handshake done, connected.");
    };

    LRWebSocketConnection.prototype._oncommand = function(command) {
      LR.log.fyi("" + this.id + ": Incoming command " + command.command);
      return this.server._onwscommand(this, command);
    };

    return LRWebSocketConnection;

  })();

  LRWebSocketServer = (function(_super) {
    __extends(LRWebSocketServer, _super);

    function LRWebSocketServer(options) {
      if (options == null) {
        options = {};
      }
      this.port = options.port || DefaultPort;
      this.connections = {};
      this.activeConnections = 0;
      this.nextConnectionId = 1;
    }

    LRWebSocketServer.prototype.start = function(callback) {
      var e;
      this.httpServer || (this.httpServer = http.createServer());
      this.httpServer.on('error', (function(_this) {
        return function(err) {
          return _this.emit('error', err);
        };
      })(this));
      try {
        return this.httpServer.listen(this.port, (function(_this) {
          return function(err) {
            if (err) {
              return callback(err);
            }
            _this.httpServer.on('request', function(request, response) {
              request.on('data', function() {});
              return request.on('end', function() {
                var url;
                url = Url.parse(request.url, true);
                return _this.emit('httprequest', url, request, response);
              });
            });
            _this.wsserver || (_this.wsserver = wsio.attach(_this.httpServer));
            _this.wsserver.on('connection', function(socket) {
              return new LRWebSocketConnection(_this, socket, "C" + (_this.nextConnectionId++));
            });
            return callback(null);
          };
        })(this));
      } catch (_error) {
        e = _error;
        return callback(e);
      }
    };

    LRWebSocketServer.prototype.monitoringConnections = function() {
      var connection, dummy, _ref1, _results;
      _ref1 = this.connections;
      _results = [];
      for (dummy in _ref1) {
        if (!__hasProp.call(_ref1, dummy)) continue;
        connection = _ref1[dummy];
        if (connection.isMonitoring()) {
          _results.push(connection);
        }
      }
      return _results;
    };

    LRWebSocketServer.prototype.monitoringConnectionCount = function() {
      return this.monitoringConnections().length;
    };

    LRWebSocketServer.prototype._onwsconnected = function(connection) {
      this.connections[connection.id] = connection;
      return this.emit('wsconnected', connection);
    };

    LRWebSocketServer.prototype._onwsdisconnected = function(connection) {
      delete this.connections[connection.id];
      return this.emit('wsdisconnected', connection);
    };

    LRWebSocketServer.prototype._onwscommand = function(connection, command) {
      return this.emit('wscommand', connection, command);
    };

    LRWebSocketServer.prototype._onwsoldproto = function(connection) {
      return this.emit('wsoldproto', connection);
    };

    return LRWebSocketServer;

  })(EventEmitter);

  module.exports = {
    LRWebSocketServer: LRWebSocketServer
  };

}).call(this);
